# Copyright © 2020 SolarWinds Worldwide, LLC. All Rights Reserved.
#
# Scripts are not supported under any SolarWinds support program or service. 
# Scripts are provided AS IS without warranty of any kind. SolarWinds further disclaims all warranties 
# including, without limitation, any implied warranties of merchantability or of fitness for a particular 
# purpose. The risk arising out of the use or performance of the scripts and documentation stays with 
# you. In no event shall SolarWinds or anyone else involved in the creation, production, or delivery of 
# the scripts be liable for any damages whatsoever (including, without limitation, damages for loss of 
# business profits, business interruption, loss of business information, or other pecuniary loss) arising 
# out of the use of or inability to use the scripts or documentation.
#
#
# Script purpose: This script helps to detect the presence of Supernova and also updates web.config to mitigate against it.
#
#
#  Version 3
#
#

#requires -RunAsAdministrator

<#
.SYNOPSIS
This script helps to detect the presence of Supernova and also updates web.config to mitigate against it.
#>

[CmdletBinding()]
param()

$ErrorActionPreference = "Stop"

$orionWebRootKey = 'HKLM:\SOFTWARE\WOW6432Node\SolarWinds.Net\SolarWinds 2002\Orion'
$orionWebRoot = Get-ItemPropertyValue $orionWebRootKey -Name 'Web Root Dir'
if ($orionWebRoot -and -not (Test-Path $orionWebRoot))
{
    Write-Host 'The Orion website location could not be found.' -ForegroundColor Red
    exit 1
}

# Build a list of paths to check
$pathsToCheck = @($orionWebRoot)

# Paths for the ASP.NET cache
$pathsToCheck += (Join-Path $env:windir 'Microsoft.NET\Framework\v2.0.50727\Temporary ASP.NET Files\root')
$pathsToCheck += (Join-Path $env:windir 'Microsoft.NET\Framework\v4.0.30319\Temporary ASP.NET Files\root')
$pathsToCheck += (Join-Path $env:windir 'Microsoft.NET\Framework64\v2.0.50727\Temporary ASP.NET Files\root')
$pathsToCheck += (Join-Path $env:windir 'Microsoft.NET\Framework64\v4.0.30319\Temporary ASP.NET Files\root')

$pathsToCheck = $pathsToCheck.Where({Test-Path $_ -PathType Container})
Write-Host 'Checking the following directories for matching files:' -ForegroundColor Yellow
Write-Host "`t$($PathsToCheck -join "$([System.Environment]::NewLine)`t")" -ForegroundColor Yellow
Write-Host

$filesToProcess = Get-ChildItem $pathsToCheck -Filter 'App_Web_logoimagehandler.ashx.b6031896.dll' -Recurse -File -Force

# check file hashes
$foundFiles = $filesToProcess |
    Where-Object {
        $SHA256 = $_ | Get-FileHash -Algorithm SHA256 | Select-Object -ExpandProperty Hash
        $SHA256 -eq 'c15abaf51e78ca56c0376522d699c978217bf041a3bd3c71d09193efa5717c71'
    }

if ($foundFiles.Count -eq 0)
{
    Write-Host 'No infected files are present.' -ForegroundColor Green
}
else
{
    Write-Host '=========== Incident Response Required ===========' -ForegroundColor Red
    Write-Host "Infected files have been found, please activate Incident Response." -ForegroundColor Red
}

# setup the folders
Write-Host
$swAppDataFolder = Join-Path $env:ProgramData 'SolarWinds'
$installersFolder = Join-Path $swAppDataFolder 'Installers'
$installerLogFolder = Join-Path $swAppDataFolder "Logs\Installer\$(Get-Date -Format 'yyyy-MM-dd_HH-mm-ss')"

New-Item $installersFolder -ItemType Directory -ErrorAction SilentlyContinue | Out-Null
New-Item $installerLogFolder -ItemType Directory -ErrorAction SilentlyContinue | Out-Null

# download and install URL Rewrite
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

Write-Host 'Installing IIS URL Rewrite' -ForegroundColor Green
$rewriteFileName = 'rewrite_amd64_en-US.msi'
$destinationInstaller = Join-Path $installersFolder rewriteFileName
Invoke-WebRequest -Uri "https://download.microsoft.com/download/1/2/8/128E2E22-C1B9-44A4-BE2A-5859ED1D4592/$rewriteFileName" -OutFile $destinationInstaller -Method Get

$installerLog = Join-Path $installerLogFolder "$rewriteFileName.log"

& $env:comspec /d /c msiexec /i "$destinationInstaller" /quiet /qn /log "$installerLog"
[bool]$succeeded = $?
[long]$errorCode = $Global:LASTEXITCODE
if (($succeeded -eq $false -and $errorCode -eq 0) -or ($errorCode -ne 0))
{
    Write-Host "Failed to execute the installer ($errorCode). Please see install log at '$installerLog'." -ForegroundColor Red
    exit 1
}

# get the web.config file
$webConfigFile = Join-Path $orionWebRoot "web.config"
$webConfigFileBackup = "$webConfigFile.$(Get-Date -Format 'yyyy-MM-dd_HH-mm-ss')"

Write-Host "Creating backup of web.config as '$webConfigFileBackup'" -ForegroundColor Green
Copy-Item -Path $webConfigFile -Destination $webConfigFileBackup -Force

$webConfigDocument = [xml](Get-Content -Path $webConfigFile -Raw)
$webServerElement = $webConfigDocument.configuration.'system.webServer'
if (-not $webServerElement)
{
    Write-Host "The Orion web.config appears invalid and is missing the 'system.webServer' element." -ForegroundColor Red
    exit 1
}

Write-Host 'Updating web.config' -ForegroundColor Green

# this is the set of rules we need to add or update
$expectedRewriteRules = [xml]@"
<rules>
    <rule name="BLockInvalidAxdRequest" patternSyntax="ECMAScript" stopProcessing="true">
        <match url="^[\s\S]+(Script|Web)Resource.axd" />
        <action type="CustomResponse" statusCode="403" statusReason="Forbidden: Access is denied." 
                statusDescription="You do not have permission to view this directory or page using the credentials that you supplied." />
    </rule>
    <rule name="PassValidi18nRequest" patternSyntax="ECMAScript" stopProcessing="true">
        <match url="^(orion|webengine).*(css|js)\.i18n\.ashx$" />
        <conditions>
            <add input="{REQUEST_METHOD}" pattern="POST" negate="true" />
        </conditions>
        <action type="None" />
    </rule>
    <rule name="BLockOtheri18nRequest" patternSyntax="ECMAScript" stopProcessing="true">
        <match url="i18n.ashx" />
        <action type="CustomResponse" statusCode="403" statusReason="Forbidden: Access is denied." 
                statusDescription="You do not have permission to view this directory or page using the credentials that you supplied." />
    </rule>
    <rule name="PassValidSkipi18nRequest" patternSyntax="ECMAScript" stopProcessing="true">
        <match url="^Orion\/Skipi18n\/Profiler\/" />
        <action type="None" />
    </rule>
    <rule name="BLockOtherSkipi18nRequest" patternSyntax="ECMAScript" stopProcessing="true">
        <match url="Skipi18n" />
        <action type="CustomResponse" statusCode="403" statusReason="Forbidden: Access is denied." 
                statusDescription="You do not have permission to view this directory or page using the credentials that you supplied." />
    </rule>
</rules>
"@

# add 'rewrite' element if not present
$rewriteElement = $webServerElement.rewrite
if (-not $rewriteElement)
{
    $rewriteElement = $webConfigDocument.CreateElement('rewrite')
    $defaultDocumentElement = $webServerElement.defaultDocument
    if ($defaultDocumentElement)
    {
        $webServerElement.InsertBefore($rewriteElement, $defaultDocumentElement) | Out-Null
    }
    else
    {
        $webServerElement.AppendChild($rewriteElement) | Out-Null
    }
}

# add 'rules' element if not present
$rulesElement = $rewriteElement.rules
if (-not $rulesElement)
{
    $rulesElement = $webConfigDocument.CreateElement('rules')
    $rewriteElement.AppendChild($rulesElement) | Out-Null
}

# replace existing rules
$expectedRuleNames = $expectedRewriteRules.rules.rule.name
@($rulesElement.rule) |
    Where-Object { $_.name -in $expectedRuleNames } |
    ForEach-Object {
        $oldRule = $_
        $newRule = $expectedRewriteRules.rules.rule.Where( { $_.name -eq $oldRule.name }, 'First')[0]
        $newRule = $webConfigDocument.ImportNode($newRule, $true)
        $rulesElement.ReplaceChild($newRule, $oldRule) | Out-Null
    }

# add missing rules
$currentRuleNames = $rulesElement.rule.name
$expectedRewriteRules.rules.rule |
    Where-Object { $_.name -notin $currentRuleNames } |
    ForEach-Object {
        $rulesElement.AppendChild($webConfigDocument.ImportNode($_, $true)) | Out-Null
    }

# save the updated web.config
$webConfigDocument.Save($webConfigFile)
Write-Host "Saved new web.config at '$webConfigFile'" -ForegroundColor Green

# SIG # Begin signature block
# MIIZrwYJKoZIhvcNAQcCoIIZoDCCGZwCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCD2/bAWCZSqP8fK
# 5HbmDpCxA1f0M8uMWlE6yPdNDhXXKKCCFKgwggT+MIID5qADAgECAhANQkrgvjqI
# /2BAIc4UAPDdMA0GCSqGSIb3DQEBCwUAMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNV
# BAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBUaW1lc3RhbXBpbmcgQ0EwHhcN
# MjEwMTAxMDAwMDAwWhcNMzEwMTA2MDAwMDAwWjBIMQswCQYDVQQGEwJVUzEXMBUG
# A1UEChMORGlnaUNlcnQsIEluYy4xIDAeBgNVBAMTF0RpZ2lDZXJ0IFRpbWVzdGFt
# cCAyMDIxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwuZhhGfFivUN
# CKRFymNrUdc6EUK9CnV1TZS0DFC1JhD+HchvkWsMlucaXEjvROW/m2HNFZFiWrj/
# ZwucY/02aoH6KfjdK3CF3gIY83htvH35x20JPb5qdofpir34hF0edsnkxnZ2OlPR
# 0dNaNo/Go+EvGzq3YdZz7E5tM4p8XUUtS7FQ5kE6N1aG3JMjjfdQJehk5t3Tjy9X
# tYcg6w6OLNUj2vRNeEbjA4MxKUpcDDGKSoyIxfcwWvkUrxVfbENJCf0mI1P2jWPo
# GqtbsR0wwptpgrTb/FZUvB+hh6u+elsKIC9LCcmVp42y+tZji06lchzun3oBc/gZ
# 1v4NSYS9AQIDAQABo4IBuDCCAbQwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQC
# MAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwQQYDVR0gBDowODA2BglghkgBhv1s
# BwEwKTAnBggrBgEFBQcCARYbaHR0cDovL3d3dy5kaWdpY2VydC5jb20vQ1BTMB8G
# A1UdIwQYMBaAFPS24SAd/imu0uRhpbKiJbLIFzVuMB0GA1UdDgQWBBQ2RIaOpLqw
# Zr68KC0dRDbd42p6vDBxBgNVHR8EajBoMDKgMKAuhixodHRwOi8vY3JsMy5kaWdp
# Y2VydC5jb20vc2hhMi1hc3N1cmVkLXRzLmNybDAyoDCgLoYsaHR0cDovL2NybDQu
# ZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC10cy5jcmwwgYUGCCsGAQUFBwEBBHkw
# dzAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME8GCCsGAQUF
# BzAChkNodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNz
# dXJlZElEVGltZXN0YW1waW5nQ0EuY3J0MA0GCSqGSIb3DQEBCwUAA4IBAQBIHNy1
# 6ZojvOca5yAOjmdG/UJyUXQKI0ejq5LSJcRwWb4UoOUngaVNFBUZB3nw0QTDhtk7
# vf5EAmZN7WmkD/a4cM9i6PVRSnh5Nnont/PnUp+Tp+1DnnvntN1BIon7h6JGA078
# 9P63ZHdjXyNSaYOC+hpT7ZDMjaEXcw3082U5cEvznNZ6e9oMvD0y0BvL9WH8dQgA
# dryBDvjA4VzPxBFy5xtkSdgimnUVQvUtMjiB2vRgorq0Uvtc4GEkJU+y38kpqHND
# Udq9Y9YfW5v3LhtPEx33Sg1xfpe39D+E68Hjo0mh+s6nv1bPull2YYlffqe0jmd4
# +TaY4cso2luHpoovMIIFMDCCBBigAwIBAgIQBAkYG1/Vu2Z1U0O1b5VQCDANBgkq
# hkiG9w0BAQsFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5j
# MRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBB
# c3N1cmVkIElEIFJvb3QgQ0EwHhcNMTMxMDIyMTIwMDAwWhcNMjgxMDIyMTIwMDAw
# WjByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQL
# ExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3Vy
# ZWQgSUQgQ29kZSBTaWduaW5nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEA+NOzHH8OEa9ndwfTCzFJGc/Q+0WZsTrbRPV/5aid2zLXcep2nQUut4/6
# kkPApfmJ1DcZ17aq8JyGpdglrA55KDp+6dFn08b7KSfH03sjlOSRI5aQd4L5oYQj
# ZhJUM1B0sSgmuyRpwsJS8hRniolF1C2ho+mILCCVrhxKhwjfDPXiTWAYvqrEsq5w
# MWYzcT6scKKrzn/pfMuSoeU7MRzP6vIK5Fe7SrXpdOYr/mzLfnQ5Ng2Q7+S1TqSp
# 6moKq4TzrGdOtcT3jNEgJSPrCGQ+UpbB8g8S9MWOD8Gi6CxR93O8vYWxYoNzQYIH
# 5DiLanMg0A9kczyen6Yzqf0Z3yWT0QIDAQABo4IBzTCCAckwEgYDVR0TAQH/BAgw
# BgEB/wIBADAOBgNVHQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwMweQYI
# KwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5j
# b20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdp
# Q2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6
# Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmww
# OqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJ
# RFJvb3RDQS5jcmwwTwYDVR0gBEgwRjA4BgpghkgBhv1sAAIEMCowKAYIKwYBBQUH
# AgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwCgYIYIZIAYb9bAMwHQYD
# VR0OBBYEFFrEuXsqCqOl6nEDwGD5LfZldQ5YMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBCwUAA4IBAQA+7A1aJLPzItEVyCx8JSl2
# qB1dHC06GsTvMGHXfgtg/cM9D8Svi/3vKt8gVTew4fbRknUPUbRupY5a4l4kgU4Q
# pO4/cY5jDhNLrddfRHnzNhQGivecRk5c/5CxGwcOkRX7uq+1UcKNJK4kxscnKqEp
# KBo6cSgCPC6Ro8AlEeKcFEehemhor5unXCBc2XGxDI+7qPjFEmifz0DLQESlE/Dm
# ZAwlCEIysjaKJAL+L3J+HNdJRZboWR3p+nRka7LrZkPas7CM1ekN3fYBIM6ZMWM9
# CBoYs4GbT8aTEAb8B4H6i9r5gkn3Ym6hU/oSlBiFLpKR6mhsRDKyZqHnGKSaZFHv
# MIIFMTCCBBmgAwIBAgIQCqEl1tYyG35B5AXaNpfCFTANBgkqhkiG9w0BAQsFADBl
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJv
# b3QgQ0EwHhcNMTYwMTA3MTIwMDAwWhcNMzEwMTA3MTIwMDAwWjByMQswCQYDVQQG
# EwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNl
# cnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgVGltZXN0
# YW1waW5nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAvdAy7kvN
# j3/dqbqCmcU5VChXtiNKxA4HRTNREH3Q+X1NaH7ntqD0jbOI5Je/YyGQmL8TvFfT
# w+F+CNZqFAA49y4eO+7MpvYyWf5fZT/gm+vjRkcGGlV+Cyd+wKL1oODeIj8O/36V
# +/OjuiI+GKwR5PCZA207hXwJ0+5dyJoLVOOoCXFr4M8iEA91z3FyTgqt30A6XLdR
# 4aF5FMZNJCMwXbzsPGBqrC8HzP3w6kfZiFBe/WZuVmEnKYmEUeaC50ZQ/ZQqLKfk
# dT66mA+Ef58xFNat1fJky3seBdCEGXIX8RcG7z3N1k3vBkL9olMqT4UdxB08r8/a
# rBD13ays6Vb/kwIDAQABo4IBzjCCAcowHQYDVR0OBBYEFPS24SAd/imu0uRhpbKi
# JbLIFzVuMB8GA1UdIwQYMBaAFEXroq/0ksuCMS1Ri6enIZ3zbcgPMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMI
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsMFAGA1UdIARJMEcwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAsGCWCGSAGG/WwH
# ATANBgkqhkiG9w0BAQsFAAOCAQEAcZUS6VGHVmnN793afKpjerN4zwY3QITvS4S/
# ys8DAv3Fp8MOIEIsr3fzKx8MIVoqtwU0HWqumfgnoma/Capg33akOpMP+LLR2HwZ
# YuhegiUexLoceywh4tZbLBQ1QwRostt1AuByx5jWPGTlH0gQGF+JOGFNYkYkh2OM
# kVIsrymJ5Xgf1gsUpYDXEkdws3XVk4WTfraSZ/tTYYmo9WuWwPRYaQ18yAGxuSh1
# t5ljhSKMYcp5lH5Z/IwP42+1ASa2bKXuh1Eh5Fhgm7oMLSttosR+u8QlK0cCCHxJ
# rhO24XxCQijGGFbPQTS2Zl22dHv1VjMiLyI2skuiSpXY9aaOUjCCBTkwggQhoAMC
# AQICEALTGDK5W7JawKuZU18w7mUwDQYJKoZIhvcNAQELBQAwcjELMAkGA1UEBhMC
# VVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0
# LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2ln
# bmluZyBDQTAeFw0yMDEyMTUwMDAwMDBaFw0yMzAxMTkyMzU5NTlaMHYxCzAJBgNV
# BAYTAlVTMQ4wDAYDVQQIEwVUZXhhczEPMA0GA1UEBxMGQXVzdGluMSIwIAYDVQQK
# ExlTb2xhcndpbmRzIFdvcmxkd2lkZSwgTExDMSIwIAYDVQQDExlTb2xhcndpbmRz
# IFdvcmxkd2lkZSwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# uXQacTQ043dXSgbGEvfKMiDT0IL9yaKxEnv31FGznzJOfHCo5eD6hMZYWhUiZcst
# 4Cy0OwhUdHeURUJjfpwUjf9TnKgTQryqZheffDepvWi2Wo3lt9OEe/l2dCAkZKxW
# cySupr7hmAhcnlNHXMw1FeP5IUGg9VNxRg0SBlmg9IH4NKZA8OXCU+aiTqqIZYSE
# qZgGt4CNQtsKAclCVSuk647ksFGNnUQB7cZFU7yoj8MGYSyUtzjZywbwtrNsXux9
# ViFwoXvqpLNBd/JaXS9QC/hFXpv4dmzn0YsLG/4ZfOeSy1TxhER+cb0ayUOqhJ7O
# SOveFzz1ig5hDnEznyciwQIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoK
# o6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFL/6v4EDCS8fFRuMurI4GAqSv19aMA4G
# A1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWg
# M6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcx
# LmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJl
# ZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcC
# ARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsG
# AQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29t
# ME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADAN
# BgkqhkiG9w0BAQsFAAOCAQEAoR5jzPm1MHxEQQaC3Jh6+gfdDi8KAmGI5ZxXAbju
# tu1urG3S0hLQw3cHg81XQZVrLfIXOLQlpjwOsg4p0IcE3l+1aaVWbdOHzbLcRF9z
# mGZrLr7xTL/PXviGD90wQft7EBsTSPO+gEVilgb7+RMCj+kf4c064TkXlOT/DmYW
# S8qoy+IWJYNvrYVkHJYtbDHfkWa9z4sMSDAofrNQaJVxIUi5Zlaagfa+tSnEqRYd
# PBL4511wxVbA+teSIcOBqZ+kevp/7YeWto7Gtkg/wAEQJ8zkZOlptGwL9T44Wfxf
# 949BROknU1PEwrx5eNJX02gBjp5oyvz8b/wgHlN5zDfRJzGCBF0wggRZAgEBMIGG
# MHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsT
# EHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJl
# ZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEALTGDK5W7JawKuZU18w7mUwDQYJYIZIAWUD
# BAIBBQCggYQwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG9w0BCQMx
# DAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkq
# hkiG9w0BCQQxIgQgCOE16MPl1+xQAmVH37rwWEAvzKxXqFKWHjJEJhBTK5IwDQYJ
# KoZIhvcNAQEBBQAEggEAIZmYldCgBWSvmhX9NQlqKjeOMsMsN3owAa+NjwiNSVk1
# zgrbNXq03KvhF86SSafhCvbe1XW2F38n7z+wfqVDtkUvVLjQmaRFe/7046vQ9jJU
# SEllowzJaMdZoAGihwNf4loeUN3cWq5snglo+FYNy4ajZ4nFEVeopvbx4wngZHsR
# wzc173DZYqMK62HOXLzurepW+kPmqbBM7lf+BvcNdn42fA07WClw/G8RdNWTqN1s
# J2yvzbTzF5Ov41mY/+ajXPr2bgyAHHbm9FyW8kOC1hZS3UM+6xV8Q/9ArrbveqBr
# jzF6ameH2UoWhnI5DX7FVeJCkT81s0ZL3oBQX6jyw6GCAiAwggIcBgkqhkiG9w0B
# CQYxggINMIICCQIBATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNl
# cnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdp
# Q2VydCBTSEEyIEFzc3VyZWQgSUQgVGltZXN0YW1waW5nIENBAhANQkrgvjqI/2BA
# Ic4UAPDdMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwG
# CSqGSIb3DQEJBTEPFw0yMTAxMDQyMDIwMjZaMCMGCSqGSIb3DQEJBDEWBBSRsbfk
# d4EhmyEXmfnAeROC78AGIjANBgkqhkiG9w0BAQEFAASCAQBYjZ/7U8GjhYUd92GW
# /YeE1oMlrUUq6evSh8Tw3QDaZl3MZkDCj1MV3WorjGGNjdjjk72WT6Hn263749TI
# R1Szy7aLrH+61NPlfFPPLEJbPzIzNv8K2AMM8SMmtB/JyuTH9lkcUQnecyhuxNps
# USewh/wTT6HltdttcSNESFPvVietbPqOFsuuTwpHJD5E03129c19j7zCAKseZ0lf
# RxOMhQjvpzFE7pG67lMRuZeN9FYi/AVDxrzR6zusGKXqq3EgcP2OrhUyq2jYEMGw
# D1ckHGBKb8N3Pw16PIux0p9DXT0lLeMz9E6AptOPvwRh+L+KNcL9e2V1/R6PiKzP
# TqPw
# SIG # End signature block
